@@ -22,11 +22,11 @@ Net::DAVTalk - Interface to talk to DAV servers
 
 =head1 VERSION
 
-Version 0.14
+Version 0.16
 
 =cut
 
-our $VERSION = '0.14';
+our $VERSION = '0.16';
 
 
 =head1 SYNOPSIS
@@ -268,10 +268,11 @@ sub Request {
     confess "Error with $Method for $URI (504, Gateway Timeout)";
   }
 
-  if ($Response->{status} == 301 or $Response->{status} == 302) {
+  my $count = 0;
+  while ($Response->{status} =~ m{^30[1278]} and (++$count < 10)) {
     my $location = URI->new_abs($Response->{headers}{location}, $URI);
     if ($ENV{DEBUGDAV}) {
-      warn "******** REDIRECT $Response->{status} to $location\n";
+      warn "******** REDIRECT ($count) $Response->{status} to $location\n";
     }
     $OldAlarm = alarm 60;
     eval {
@@ -357,6 +358,19 @@ perform a propfind on a particular path and get the properties back
 
 sub GetProps {
   my ($Self, $Path, @Props) = @_;
+  my @res = $Self->GetPropsArray($Path, @Props);
+  return wantarray ? map { $_->[0] } @res : $res[0][0];
+}
+
+=head2 $Self->GetPropsArray($Path, @Props)
+
+perform a propfind on a particular path and get the properties back
+as an array of one or more items
+
+=cut
+
+sub GetPropsArray {
+  my ($Self, $Path, @Props) = @_;
 
   # Fetch one or more properties.
   #  Use [ 'prop', 'sub', 'item' ] to dig into result structure
@@ -379,21 +393,30 @@ sub GetProps {
     foreach my $Propstat (@{$Response->{"{$NS_D}propstat"} || []}) {
       my $PropData = $Propstat->{"{$NS_D}prop"} || next;
       for my $Prop (@Props) {
-        my $Result = $PropData;
+        my @Values = ($PropData);
 
         # Array ref means we need to git through structure
-        for (ref $Prop ? @$Prop : $Prop) {
-          last if !$Result;
-          if (/:/) {
-            my ($N, $P) = split /:/, $_;
-            my $NS = $Self->ns($N);
-            $Result = $Result->{"{$NS}$P"};
-          } else {
-            $Result = $Result->{$_};
+        foreach my $Key (ref $Prop ? @$Prop : $Prop) {
+          my @New;
+          foreach my $Result (@Values) {
+            if ($Key =~ m/:/) {
+              my ($N, $P) = split /:/, $Key;
+              my $NS = $Self->ns($N);
+              $Result = $Result->{"{$NS}$P"};
+            } else {
+              $Result = $Result->{$Key};
+            }
+            if (ref($Result) eq 'ARRAY') {
+              push @New, @$Result;
+            }
+            elsif (defined $Result) {
+              push @New, $Result;
+            }
           }
+          @Values = @New;
         }
-        $Result = $Result ? $Result->{content} : undef;
-        push @Results, $Result;
+
+        push @Results, [ map { $_->{content} } @Values ];
       }
     }
   }
